//===- Hello.cpp - Example code from "Writing an LLVM Pass" ---------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This file implements two versions of the LLVM "Hello World" pass described
// in docs/WritingAnLLVMPass.html
//
//===----------------------------------------------------------------------===//

#define DEBUG_TYPE "hello"
#include "llvm/Pass.h"
#include "llvm/IR/Function.h"
#include "llvm/ADT/Statistic.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/IR/InstIterator.h"
#include "llvm/IR/Instruction.h"
#include "llvm/Analysis/ConstantFolding.h"
#include "llvm/Analysis/TargetLibraryInfo.h"
#include <set>

using namespace llvm;
STATISTIC(HelloCounter, "Counts number of functions greeted");

namespace {
// Hello - The first implementation, without getAnalysisUsage.
struct Hello: public FunctionPass {
	static char ID; // Pass identification, replacement for typeid
	Hello() :
		FunctionPass(ID) {
	}

	bool runOnFunction(Function &F) override {
		++HelloCounter;
		DEBUG(dbgs() << "Hello - Const Prop : ");
		DEBUG(dbgs().write_escaped(F.getName()) << '\n');

		std::set<Instruction*> WorkList;

        //TODO: Parse all the instructions and add them to WorkList :: hint: inst_begin, inst_end

		bool Changed = false;
        const DataLayout &DL = F.getParent()->getDataLayout();

        //TODO: extract one instruction at a time from the pool, until is empty
        //TODO: check if it is not dead instruction
        //TODO: check if can be applied constant folding :: hint: ConstantFoldInstruction
        //TODO: parse all the users of the instruction and add them to the worklist
        //TODO: modify the users with the constant (hint: replaceAllUses...), remove the instruction (from the worklist && from the block)
        //TODO: update the changed bool accordingly


		return Changed;
	}

	virtual void getAnalysisUsage(AnalysisUsage &AU) const {
		AU.setPreservesCFG();
	}

};
}

char Hello::ID = 0;
static RegisterPass<Hello> X("hello", "Hello World Pass");



